@@ -81,14 +81,17 @@ private Q_SLOTS:
     void initTestCase()
     {
         QStandardPaths::setTestModeEnabled(true);
-        qputenv("KIOSLAVE_ENABLE_TESTMODE", "1"); // ensure the KIO workers call QStandardPaths::setTestModeEnabled too
+        qputenv("KIOWORKER_ENABLE_TESTMODE", "1"); // ensure the KIO workers call QStandardPaths::setTestModeEnabled too
 
-        KIO::setDefaultJobUiDelegateFactoryV2(nullptr);
+        KIO::setDefaultJobUiDelegateFactory(nullptr);
         KIO::setDefaultJobUiDelegateExtension(nullptr);
 
         m_trashDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/Trash");
         QDir(m_trashDir).removeRecursively();
 
+        m_tempDir = QTemporaryDir(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/dropjobtest"));
+        m_nonWritableTempDir = QTemporaryDir(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/dropjobtest_ro"));
+
         QVERIFY(m_tempDir.isValid());
         QVERIFY(m_nonWritableTempDir.isValid());
         QVERIFY(QFile(m_nonWritableTempDir.path()).setPermissions(QFile::ReadOwner | QFile::ReadUser | QFile::ExeOwner | QFile::ExeUser));
@@ -199,7 +202,7 @@ private Q_SLOTS:
         const QUrl destUrl = QUrl::fromLocalFile(dest);
         m_mimeData.setUrls(QList<QUrl>{QUrl::fromLocalFile(srcFile)});
         QDropEvent dropEvent(QPoint(10, 10), dropAction, &m_mimeData, Qt::LeftButton, modifiers);
-        KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo | KIO::NoPrivilegeExecution);
+        KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
         JobSpy jobSpy(job);
         QSignalSpy copyJobSpy(job, &KIO::DropJob::copyJobStarted);
         QSignalSpy itemCreatedSpy(job, &KIO::DropJob::itemCreated);
@@ -368,6 +371,36 @@ private Q_SLOTS:
         QCOMPARE(job->error(), KIO::ERR_DROP_ON_ITSELF);
     }
 
+// Test this only on Linux due to using xdg filepath
+#ifdef Q_OS_LINUX
+    void shouldDropFromXdgTrashToTrash() // #497390
+    {
+        // Given a file in the trash
+        const QString xdgtrash = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/Trash/files/srcfile");
+        // We utilize the file from previous test, but create the file if it doesn't exist
+        if (!QFile::exists(xdgtrash)) {
+            QVERIFY(QFileInfo(m_srcFile).isWritable());
+            KIO::CopyJob *copyJob = KIO::move(QUrl::fromLocalFile(m_srcFile), QUrl(QStringLiteral("trash:/")));
+            QSignalSpy copyingDoneSpy(copyJob, &KIO::CopyJob::copyingDone);
+            QVERIFY(copyJob->exec());
+            const QUrl trashUrl = copyingDoneSpy.at(0).at(2).value<QUrl>();
+            QVERIFY(trashUrl.isValid());
+            QVERIFY(!QFile::exists(m_srcFile));
+        }
+
+        // When dropping the trashed file in the trash
+        m_mimeData.setUrls(QList<QUrl>{QUrl::fromLocalFile(xdgtrash)});
+        QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
+        KIO::DropJob *job = KIO::drop(&dropEvent, QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
+        QSignalSpy copyJobSpy(job, &KIO::DropJob::copyJobStarted);
+        QSignalSpy spy(job, &KIO::DropJob::itemCreated);
+
+        // Then an error should be reported and no files action should occur
+        QVERIFY(!job->exec());
+        QCOMPARE(job->error(), KIO::ERR_DROP_ON_ITSELF);
+    }
+#endif
+
     void shouldDropToDirectoryWithPopup_data()
     {
         QTest::addColumn<QString>("dest"); // empty for a temp dir
@@ -405,7 +438,7 @@ private Q_SLOTS:
 
         // When dropping the source file onto the directory
         QUrl destUrl = QUrl::fromLocalFile(dest);
-        QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction /*unused*/, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
+        QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction | Qt::MoveAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
         JobSpy jobSpy(job);
         qRegisterMetaType<KFileItemListProperties>();
@@ -417,14 +450,15 @@ private Q_SLOTS:
         QVERIFY(spyShow.wait());
         QTRY_VERIFY(findPopup());
         QMenu *popup = findPopup();
+
         QCOMPARE(int(popupDropActions(popup)), int(offeredActions));
 
         // And when selecting action number <triggerActionNumber>
         QAction *action = popup->actions().at(triggerActionNumber);
         QVERIFY(action);
         QCOMPARE(int(action->data().value<Qt::DropAction>()), int(expectedDropAction));
-        const QRect actionGeom = popup->actionGeometry(action);
-        QTest::mouseClick(popup, Qt::LeftButton, Qt::NoModifier, actionGeom.center());
+        action->trigger();
+        popup->hide();
 
         // Then the job should finish, and the chosen action should happen.
         QVERIFY(jobSpy.waitForResult());
@@ -449,7 +483,9 @@ private Q_SLOTS:
         const QUrl destUrl = QUrl::fromLocalFile(tempDestDir.path());
 
         // When dropping the source file onto the directory
-        QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction /*unused*/, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
+        // here we must use Qt::MoveAction because otherwise
+        // application (plugins) actions are not add to drop popup menu
+        QDropEvent dropEvent(QPoint(10, 10), Qt::MoveAction /*unused*/, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
         QAction appAction1(QStringLiteral("action1"), this);
         QAction appAction2(QStringLiteral("action2"), this);
@@ -464,12 +500,22 @@ private Q_SLOTS:
         const QList<QAction *> actions = popup->actions();
         QVERIFY(actions.contains(&appAction1));
         QVERIFY(actions.contains(&appAction2));
+
         QVERIFY(actions.at(actions.indexOf(&appAction1) - 1)->isSeparator());
-        QVERIFY(actions.at(actions.indexOf(&appAction2) + 1)->isSeparator());
+
+        // there is at least one separator after action2
+        int sepIndex = actions.indexOf(&appAction2) + 1;
+        while (sepIndex < actions.length()) {
+            if (actions.at(sepIndex)->isSeparator()) {
+                break;
+            }
+            ++sepIndex;
+        }
+        QVERIFY(sepIndex < actions.length());
 
         // And when selecting action appAction1
-        const QRect actionGeom = popup->actionGeometry(&appAction1);
-        QTest::mouseClick(popup, Qt::LeftButton, Qt::NoModifier, actionGeom.center());
+        appAction1.trigger();
+        popup->hide();
 
         // Then the menu should hide and the job terminate (without doing any copying)
         QVERIFY(jobSpy.waitForResult());
