@@ -35,11 +35,6 @@ static inline Worker *jobSWorker(SimpleJob *job)
     return SimpleJobPrivate::get(job)->m_worker;
 }
 
-static inline int jobCommand(SimpleJob *job)
-{
-    return SimpleJobPrivate::get(job)->m_command;
-}
-
 static inline void startJob(SimpleJob *job, Worker *worker)
 {
     SimpleJobPrivate::get(job)->start(worker);
@@ -71,7 +66,6 @@ public:
     bool m_ignoreConfigReparse = false;
 
     void doJob(SimpleJob *job);
-    void setJobPriority(SimpleJob *job, int priority);
     void cancelJob(SimpleJob *job);
     void jobFinished(KIO::SimpleJob *job, KIO::Worker *worker);
     void putWorkerOnHold(KIO::SimpleJob *job, const QUrl &url);
@@ -122,14 +116,6 @@ Scheduler *scheduler()
 
 ////////////////////////////
 
-int SerialPicker::changedPrioritySerial(int oldSerial, int newPriority) const
-{
-    Q_ASSERT(newPriority >= -10 && newPriority <= 10);
-    newPriority = qBound(-10, newPriority, 10);
-    int unbiasedSerial = oldSerial % m_jobsPerPriority;
-    return unbiasedSerial + newPriority * m_jobsPerPriority;
-}
-
 WorkerManager::WorkerManager()
 {
     m_grimTimer.setSingleShot(true);
@@ -289,7 +275,7 @@ static void ensureNoDuplicates(QMap<int, HostQueue *> *queuesBySerial)
 #endif
 }
 
-static void verifyRunningJobsCount(QHash<QString, HostQueue> *queues, int runningJobsCount)
+static void verifyRunningJobsCount(const std::unordered_map<QString, HostQueue> *const queues, int runningJobsCount)
 {
     Q_UNUSED(queues);
     Q_UNUSED(runningJobsCount);
@@ -297,7 +283,7 @@ static void verifyRunningJobsCount(QHash<QString, HostQueue> *queues, int runnin
     int realRunningJobsCount = 0;
     auto it = queues->cbegin();
     for (; it != queues->cend(); ++it) {
-        realRunningJobsCount += it.value().runningJobsCount();
+        realRunningJobsCount += it->second.runningJobsCount();
     }
     Q_ASSERT(realRunningJobsCount == runningJobsCount);
 
@@ -305,7 +291,7 @@ static void verifyRunningJobsCount(QHash<QString, HostQueue> *queues, int runnin
     QSet<SimpleJob *> seenJobs;
     auto it2 = queues->cbegin();
     for (; it2 != queues->cend(); ++it2) {
-        for (SimpleJob *job : it2.value().runningJobs()) {
+        for (SimpleJob *job : it2->second.runningJobs()) {
             Q_ASSERT(!seenJobs.contains(job));
             seenJobs.insert(job);
         }
@@ -378,29 +364,6 @@ void ProtoQueue::queueJob(SimpleJob *job)
     ensureNoDuplicates(&m_queuesBySerial);
 }
 
-void ProtoQueue::changeJobPriority(SimpleJob *job, int newPrio)
-{
-    SimpleJobPrivate *jobPriv = SimpleJobPrivate::get(job);
-    QHash<QString, HostQueue>::Iterator it = m_queuesByHostname.find(jobPriv->m_url.host());
-    if (it == m_queuesByHostname.end()) {
-        return;
-    }
-    HostQueue &hq = it.value();
-    const int prevLowestSerial = hq.lowestSerial();
-    if (hq.isJobRunning(job) || !hq.removeJob(job)) {
-        return;
-    }
-    jobPriv->m_schedSerial = m_serialPicker.changedPrioritySerial(jobPriv->m_schedSerial, newPrio);
-    hq.queueJob(job);
-    const bool needReinsert = hq.lowestSerial() != prevLowestSerial;
-    // the host queue might be absent from m_queuesBySerial because the connections per host limit
-    // for that host has been reached.
-    if (needReinsert && m_queuesBySerial.remove(prevLowestSerial)) {
-        m_queuesBySerial.insert(hq.lowestSerial(), &hq);
-    }
-    ensureNoDuplicates(&m_queuesBySerial);
-}
-
 void ProtoQueue::removeJob(SimpleJob *job)
 {
     SimpleJobPrivate *jobPriv = SimpleJobPrivate::get(job);
@@ -434,7 +397,7 @@ void ProtoQueue::removeJob(SimpleJob *job)
 
         if (hq.isEmpty()) {
             // no queued jobs, no running jobs. this destroys hq from above.
-            m_queuesByHostname.remove(jobPriv->m_url.host());
+            m_queuesByHostname.erase(jobPriv->m_url.host());
         }
 
         if (jobPriv->m_worker && jobPriv->m_worker->isAlive()) {
@@ -476,7 +439,7 @@ QList<Worker *> ProtoQueue::allWorkers() const
     QList<Worker *> ret(m_workerManager.allWorkers());
     auto it = m_queuesByHostname.cbegin();
     for (; it != m_queuesByHostname.cend(); ++it) {
-        ret.append(it.value().allWorkers());
+        ret.append(it->second.allWorkers());
     }
 
     return ret;
@@ -492,7 +455,7 @@ void ProtoQueue::startAJob()
     // qDebug() << "m_runningJobsCount:" << m_runningJobsCount;
     auto it = m_queuesByHostname.cbegin();
     for (; it != m_queuesByHostname.cend(); ++it) {
-        const QList<KIO::SimpleJob *> list = it.value().runningJobs();
+        const QList<KIO::SimpleJob *> list = it->second.runningJobs();
         for (SimpleJob *job : list) {
             // qDebug() << SimpleJobPrivate::get(job)->m_url;
         }
@@ -684,16 +647,6 @@ void SchedulerPrivate::doJob(SimpleJob *job)
     proto->queueJob(job);
 }
 
-void SchedulerPrivate::setJobPriority(SimpleJob *job, int priority)
-{
-    // qDebug() << job << priority;
-    const QString protocol = SimpleJobPrivate::get(job)->m_protocol;
-    if (!protocol.isEmpty()) {
-        ProtoQueue *proto = protoQ(protocol, job->url().host());
-        proto->changeJobPriority(job, priority);
-    }
-}
-
 void SchedulerPrivate::cancelJob(SimpleJob *job)
 {
     KIO::SimpleJobPrivate *const jobPriv = SimpleJobPrivate::get(job);
