@@ -14,7 +14,6 @@
 
 #include <QDataStream>
 #include <QDebug>
-#include <QList>
 #include <QString>
 
 #include <KUser>
@@ -22,7 +21,6 @@
 using namespace KIO;
 
 // BEGIN UDSEntryPrivate
-/* ---------- UDSEntryPrivate ------------ */
 
 class KIO::UDSEntryPrivate : public QSharedData
 {
@@ -41,9 +39,9 @@ public:
     void save(QDataStream &s) const;
     void load(QDataStream &s);
     void debugUDSEntry(QDebug &stream) const;
-    /**
-     * @param field numeric UDS field id
-     * @return the name of the field
+    /*
+     * \a field numeric UDS field id
+     * Returns the name of the field
      */
     static QString nameOfUdsField(uint field);
 
@@ -127,7 +125,7 @@ void UDSEntryPrivate::replace(uint udsField, long long value)
 
 int UDSEntryPrivate::count() const
 {
-    return storage.size();
+    return int(storage.size());
 }
 
 QString UDSEntryPrivate::stringValue(uint udsField) const
@@ -209,6 +207,13 @@ void UDSEntryPrivate::load(QDataStream &s)
         cachedStrings.resize(size);
     }
 
+    // We cache the string buffer instance, increasing the capacity by usage,
+    // only change the size on it.
+    // Relying on (undocumented/unspecified) behaviour of
+    // operator>>(QDataStream &stream, QString &string)
+    // to reuse existing allocation.
+    thread_local QString buffer;
+
     for (quint32 i = 0; i < size; ++i) {
         quint32 uds;
         s >> uds;
@@ -217,14 +222,14 @@ void UDSEntryPrivate::load(QDataStream &s)
             // If the QString is the same like the one we read for the
             // previous UDSEntry at the i-th position, use an implicitly
             // shared copy of the same QString to save memory.
-            QString buffer;
             s >> buffer;
 
-            if (buffer != cachedStrings.at(i)) {
-                cachedStrings[i] = buffer;
+            QString &cachedString = cachedStrings[i];
+            if (buffer != cachedString) {
+                cachedString = buffer;
             }
 
-            insert(uds, cachedStrings.at(i));
+            insert(uds, cachedString);
         } else if (uds & KIO::UDSEntry::UDS_NUMBER) {
             long long value;
             s >> value;
@@ -352,7 +357,7 @@ UDSEntry::UDSEntry(const QT_STATBUF &buff, const QString &name)
     d->insert(UDS_ACCESS_TIME, buff.st_atime);
 #ifndef Q_OS_WIN
     d->insert(UDS_LOCAL_USER_ID, buff.st_uid);
-    d->insert(UDS_LOCAL_GROUP_ID,  buff.st_gid);
+    d->insert(UDS_LOCAL_GROUP_ID, buff.st_gid);
 #endif
 }
 
@@ -446,7 +451,21 @@ KIOCORE_EXPORT QDataStream &operator>>(QDataStream &s, KIO::UDSEntry &a)
     return s;
 }
 
+// TODO KF7 remove
+// legacy operator in global namespace for binary compatibility
 KIOCORE_EXPORT bool operator==(const KIO::UDSEntry &entry, const KIO::UDSEntry &other)
+{
+    return KIO::operator==(entry, other);
+}
+
+// TODO KF7 remove
+// legacy operator in global namespace for binary compatibility
+KIOCORE_EXPORT bool operator!=(const KIO::UDSEntry &entry, const KIO::UDSEntry &other)
+{
+    return KIO::operator!=(entry, other);
+}
+
+bool KIO::operator==(const KIO::UDSEntry &entry, const KIO::UDSEntry &other)
 {
     if (entry.count() != other.count()) {
         return false;
@@ -472,7 +491,7 @@ KIOCORE_EXPORT bool operator==(const KIO::UDSEntry &entry, const KIO::UDSEntry &
     return true;
 }
 
-KIOCORE_EXPORT bool operator!=(const KIO::UDSEntry &entry, const KIO::UDSEntry &other)
+bool KIO::operator!=(const KIO::UDSEntry &entry, const KIO::UDSEntry &other)
 {
-    return !(entry == other);
+    return !KIO::operator==(entry, other);
 }
